import 'dart:ui';
import 'dart:math';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class DefaultInputDecoration extends InputDecoration {
  DefaultInputDecoration(
      {String? placeHolder, Widget? suffixIcon, String errorText = ''})
      : super(
            fillColor: Colors.white,
            filled: true,
            hintText: placeHolder,
            hintStyle:
                const TextStyle(color: Color.fromARGB(255, 194, 198, 205)),
            contentPadding: const EdgeInsets.fromLTRB(12, 8, 12, 8),
            border: OutlineInputBorder(
                borderRadius: BorderRadius.circular(4.0),
                borderSide: const BorderSide(
                    color: Color.fromARGB(255, 233, 234, 239), width: 2)),
            enabledBorder: OutlineInputBorder(
                //未选中时候的颜色
                borderRadius: BorderRadius.circular(4.0),
                borderSide: const BorderSide(
                    color: Color.fromARGB(255, 225, 227, 233), width: 2)),
            focusedBorder: OutlineInputBorder(
                //选中时外边框颜色
                borderRadius: BorderRadius.circular(5.0),
                borderSide: const BorderSide(
                    color: Color.fromARGB(255, 97, 174, 254), width: 2)),
            suffixIcon: suffixIcon,
            errorText: errorText == '' ? null : errorText);
}

class STextField extends TextField {
  STextField(
      {Key? key,
      TextEditingController? controller,
      List<TextInputFormatter>? inputFormatters,
      String? placeHolder,
      ValueChanged<String>? onChanged,
      bool obscureText = false,
      int maxLines = 1,
      ValueChanged<String>? onSubmitted,
      Widget? suffixIcon,
      String errorText = ''})
      : super(
            key: key,
            controller: controller,
            inputFormatters: inputFormatters,
            maxLines: maxLines,
            style: const TextStyle(color: Color.fromARGB(255, 96, 98, 102)),
            onChanged: onChanged,
            obscureText: obscureText,
            onSubmitted: onSubmitted,
            decoration: DefaultInputDecoration(
                placeHolder: placeHolder,
                suffixIcon: suffixIcon,
                errorText: errorText));
}

class SButton extends ElevatedButton {
  const SButton(
      {Key? key, required VoidCallback? onPressed, required Widget? child})
      : super(key: key, onPressed: onPressed, child: child);
}

class SIconButtonChild extends StatelessWidget {
  const SIconButtonChild({Key? key, required this.label, required this.icon})
      : super(key: key);

  final Widget label;
  final Widget icon;

  @override
  Widget build(BuildContext context) {
    final double scale = MediaQuery.maybeOf(context)?.textScaleFactor ?? 1;
    final double gap = scale <= 1 ? 8 : lerpDouble(8, 4, min(scale - 1, 1))!;
    return Row(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[icon, SizedBox(width: gap), Flexible(child: label)],
    );
  }
}

class SComButton extends ElevatedButton {
  SComButton(
      {Key? key,
      required VoidCallback? onPressed,
      required Widget icon,
      required Widget label})
      : super(
            key: key,
            onPressed: onPressed,
            child: SIconButtonChild(icon: icon, label: label));

  @override
  ButtonStyle defaultStyleOf(BuildContext context) {
    final EdgeInsetsGeometry scaledPadding = ButtonStyleButton.scaledPadding(
      const EdgeInsetsDirectional.fromSTEB(12, 0, 16, 0),
      const EdgeInsets.symmetric(horizontal: 8),
      const EdgeInsetsDirectional.fromSTEB(8, 0, 4, 0),
      MediaQuery.maybeOf(context)?.textScaleFactor ?? 1,
    );
    return super.defaultStyleOf(context).copyWith(
          padding: MaterialStateProperty.all<EdgeInsetsGeometry>(scaledPadding),
        );
  }
}

class SIconButton extends IconButton {
  const SIconButton(
      {Key? key,
      required String toolTip,
      required VoidCallback? onPressed,
      required Widget icon})
      : super(
            key: key,
            onPressed: onPressed,
            icon: icon,
            splashRadius: 1,
            tooltip: toolTip);
}

typedef DayCellBuilder = Widget Function(DateTime day);

class SCalendarView extends StatelessWidget {
  final DateTime start;
  final DateTime end;
  final DayCellBuilder cellBuilder;
  final List<DateTime> _days = [];
  SCalendarView(
      {super.key,
      required this.start,
      required this.end,
      required this.cellBuilder}) {
    var dayCount = end.difference(start).inDays;
    for (var i = 0; i < dayCount; i++) {
      _days.add(start.add(Duration(days: i)));
    }
  }

  Widget _createWeekTitle() {
    return Container(
        color: Colors.white,
        child: Row(
          children: [
            Expanded(
                child: Container(
              alignment: Alignment.bottomCenter,
              child: const Text('周一', textAlign: TextAlign.center),
            )),
            Expanded(
                child: Container(
              alignment: Alignment.bottomCenter,
              child: const Text('周二', textAlign: TextAlign.center),
            )),
            Expanded(
                child: Container(
              alignment: Alignment.bottomCenter,
              child: const Text('周三', textAlign: TextAlign.center),
            )),
            Expanded(
                child: Container(
              alignment: Alignment.bottomCenter,
              child: const Text('周四', textAlign: TextAlign.center),
            )),
            Expanded(
                child: Container(
              alignment: Alignment.bottomCenter,
              child: const Text('周五', textAlign: TextAlign.center),
            )),
            Expanded(
                child: Container(
              alignment: Alignment.bottomCenter,
              child: const Text('周六', textAlign: TextAlign.center),
            )),
            Expanded(
                child: Container(
              alignment: Alignment.bottomCenter,
              child: const Text('周日', textAlign: TextAlign.center),
            )),
          ],
        ));
  }

  Widget _createDayCell(DateTime? day) {
    return day == null
        ? Container(color: Colors.white)
        : Container(color: Colors.white, child: cellBuilder(day));
  }

  Widget _createLine(int line) {
    var missingBeforeDays = start.weekday - 1;
    var startIndex = line * 7;
    List<Widget> dayCells = [];
    List<DateTime> lineDays = [];
    if (line == 0) {
      lineDays = _days.take(7 - missingBeforeDays).toList();
      var missAfterDays = 7 - lineDays.length - missingBeforeDays;
      for (var i = 0; i < missingBeforeDays; i++) {
        dayCells.add(Expanded(child: _createDayCell(null)));
        dayCells.add(Container(width: 0.5, color: Colors.grey));
      }
      for (var i = 0; i < lineDays.length; i++) {
        dayCells.add(Expanded(child: _createDayCell(lineDays[i])));
        if (i != lineDays.length - 1 || missAfterDays > 0) {
          dayCells.add(Container(width: 0.5, color: Colors.grey));
        }
      }
      if (missAfterDays > 0) {
        for (var i = 0; i < missAfterDays; i++) {
          dayCells.add(Expanded(child: _createDayCell(null)));
          if (i != missAfterDays - 1) {
            dayCells.add(Container(width: 0.5, color: Colors.grey));
          }
        }
      }
    } else {
      lineDays = _days.skip(startIndex - missingBeforeDays).take(7).toList();
      for (var i = 0; i < lineDays.length; i++) {
        dayCells.add(Expanded(child: _createDayCell(lineDays[i])));
        if (i != lineDays.length - 1 || lineDays.length < 7) {
          dayCells.add(Container(width: 0.5, color: Colors.grey));
        }
      }
      var missCellNum = 7 - lineDays.length;
      if (missCellNum > 0) {
        for (var i = 0; i < missCellNum; i++) {
          dayCells.add(Expanded(child: _createDayCell(null)));
          if (i != missCellNum - 1) {
            dayCells.add(Container(width: 0.5, color: Colors.grey));
          }
        }
      }
    }
    return Row(children: dayCells);
  }

  @override
  Widget build(BuildContext context) {
    var dayCount = end.difference(start).inDays;
    var missingDays = start.weekday - 1;
    var lineCount = ((dayCount + missingDays) / 7).ceil();
    List<Widget> lines = [];
    lines.add(_createWeekTitle());
    lines.add(Container(height: 1, color: Colors.grey));
    for (var i = 0; i < lineCount; i++) {
      lines.add(Expanded(child: _createLine(i)));
      lines.add(Container(height: 0.5, color: Colors.grey));
    }
    return Column(children: lines);
  }
}
