import 'package:flutter/material.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:sss/controllers.dart';
import 'package:sss/employeelist.dart';
import 'package:sss/holiday.dart';
import 'package:sss/dashboard.dart';
import 'package:sss/setting.dart';
import 'package:sss/statistics.dart';
import 'package:sss/tootls.dart';
import 'package:sss/viewstatistics.dart';
import 'package:url_strategy/url_strategy.dart';

import 'common.dart';
import 'constants.dart';
import 'employee.dart';
import 'layout.dart';
import 'login.dart';
import 'employeestatistics.dart';

TransitionBuilder initLoading({
  TransitionBuilder? builder,
}) {
  return (BuildContext context, Widget? child) {
    EasyLoading.instance.loadingStyle = EasyLoadingStyle.custom;
    EasyLoading.instance
      ..loadingStyle = EasyLoadingStyle.custom
      ..indicatorType = EasyLoadingIndicatorType.fadingCube
      ..indicatorSize = 32.0
      ..radius = 8.0
      ..backgroundColor = Colors.white
      ..indicatorColor = Colors.green
      ..textColor = Colors.black
      ..userInteractions = false
      ..maskType = EasyLoadingMaskType.black
      ..progressColor = Colors.green
      ..progressWidth = 4.0
      ..dismissOnTap = false;
    return FlutterEasyLoading(child: child);
  };
}

void main() {
  setPathUrlStrategy();
  Common.init();
  ControllerManager.init();
  runApp(GetMaterialApp(
      localizationsDelegates: GlobalMaterialLocalizations.delegates,
      supportedLocales: const [Locale('zh', 'CH'), Locale('en', 'US')],
      theme: ThemeData(
          fontFamily: 'msyh',
          primarySwatch: Common.createMaterialColor(Colors.green)),
      initialRoute: '/',
      getPages: [
        GetPage(
            name: '/',
            page: () => const MainPage(),
            transition: Transition.rightToLeft),
        GetPage(
            name: '/viewstatistcs',
            page: () => ViewStatisticsPage(),
            transition: Transition.rightToLeft),
        GetPage(
            name: '/viewworkingdays',
            page: () => EmployeeStatisticsPage(),
            transition: Transition.rightToLeft),
        GetPage(
            name: '/employeelist',
            page: () => EmployeeListPage(),
            transition: Transition.rightToLeft),
        GetPage(name: '/tools', page: () => ToolsPage()),
      ],
      builder: initLoading()));
}

class MainView extends StatelessWidget {
  PageController pageController;
  final Map<String, Widget> _desktopModuleMap = <String, Widget>{};
  final List<String> _moduleNames = [
    ModuleNames.employee,
    ModuleNames.holiday,
    ModuleNames.statistics,
    ModuleNames.setting,
    ModuleNames.dashBoard
  ];
  MainView({super.key, required this.pageController}) {
    var employeeModule = EmployeeModule();
    _desktopModuleMap[ModuleNames.employee] = employeeModule;
    _desktopModuleMap[ModuleNames.holiday] = const HolidayPage();
    _desktopModuleMap[ModuleNames.statistics] = const StatisticsModule();
    _desktopModuleMap[ModuleNames.setting] = const SettingePage();
    _desktopModuleMap[ModuleNames.dashBoard] = DashboardModule();
  }

  Widget _getDesktopModule(String moduleName) {
    return _desktopModuleMap[moduleName] ?? Container();
  }

  IconData _getMenuIcon(String menuName) {
    switch (menuName) {
      case ModuleNames.employee:
        return Icons.person;
      case ModuleNames.holiday:
        return Icons.card_travel;
      case ModuleNames.statistics:
        return Icons.calendar_month;
      case ModuleNames.setting:
        return Icons.settings;
      case ModuleNames.dashBoard:
        return Icons.dashboard;
      default:
        return Icons.rectangle;
    }
  }

  String _getMenuTitle(String menuName) {
    switch (menuName) {
      case ModuleNames.employee:
        return '人员管理';
      case ModuleNames.holiday:
        return '假期管理';
      case ModuleNames.statistics:
        return '工时统计';
      case ModuleNames.setting:
        return '系统设置';
      case ModuleNames.dashBoard:
        return '统计面板';
      default:
        return '';
    }
  }

  Widget _createMenu(MainController controller, String menuName) {
    var menuTitle = _getMenuTitle(menuName);
    return Obx(() => ListTile(
          contentPadding: const EdgeInsets.all(10.0),
          leading: CircleAvatar(child: Icon(_getMenuIcon(menuName))),
          title: Text(menuTitle),
          selected: menuName == controller.currentModule.value,
          onTap: () {
            controller.showModule(menuName);
          },
        ));
  }

  Widget _createMolduleMenus(MainController controller) {
    return ListView(
      children: [
        _createMenu(controller, ModuleNames.employee),
        _createMenu(controller, ModuleNames.holiday),
        _createMenu(controller, ModuleNames.statistics),
        _createMenu(controller, ModuleNames.setting),
        _createMenu(controller, ModuleNames.dashBoard),
      ],
    );
  }

  BottomNavigationBarItem _createNavigationBarItem(String menuName) {
    return BottomNavigationBarItem(
        icon: Icon(_getMenuIcon(menuName)), label: _getMenuTitle(menuName));
  }

  List<BottomNavigationBarItem> _createNavigationBarItems() {
    return [
      _createNavigationBarItem(ModuleNames.employee),
      _createNavigationBarItem(ModuleNames.holiday),
      _createNavigationBarItem(ModuleNames.statistics),
      _createNavigationBarItem(ModuleNames.setting),
      _createNavigationBarItem(ModuleNames.dashBoard),
    ];
  }

  Widget _createDesktopMainArea(MainController controller) {
    return Row(children: [
      SizedBox(
        width: 192,
        child: _createMolduleMenus(controller),
      ),
      Container(width: 0.5, color: Colors.grey),
      Expanded(
          child: Obx(() => _getDesktopModule(controller.currentModule.value)))
    ]);
  }

  Widget _createMobileMainArea(MainController controller) {
    return Obx(() => _getDesktopModule(controller.currentModule.value));
  }

  Widget _desktopBuild(context) {
    var controller = ControllerManager.find<MainController>();
    return Scaffold(
        resizeToAvoidBottomInset: false,
        appBar: AppBar(
            toolbarHeight: 48,
            title: const Text('VINNO工时统计系统'),
            actions: [
              IconButton(
                  onPressed: () async {
                    await controller.logOff();
                    pageController.animateToPage(0,
                        duration: const Duration(milliseconds: 300),
                        curve: Curves.linear);
                  },
                  icon: const Icon(Icons.exit_to_app))
            ]),
        body: SizedBox(
          width: LayoutUtils.width.value,
          height: LayoutUtils.height.value,
          child: LayoutUtils.width < 1280
              ? FittedBox(
                  fit: BoxFit.fitWidth,
                  alignment: Alignment.topLeft,
                  child: SizedBox(
                    width: LayoutUtils.layoutWidth,
                    height: LayoutUtils.layoutWidth / LayoutUtils.aspectRatio,
                    child: _createDesktopMainArea(controller),
                  ))
              : _createDesktopMainArea(controller),
        ));
  }

  Widget _mobileBuild(context) {
    var controller = ControllerManager.find<MainController>();
    return Scaffold(
      appBar: AppBar(title: const Text('VINNO工时统计系统'), actions: [
        IconButton(
            onPressed: () => controller.logOff(),
            icon: const Icon(Icons.exit_to_app))
      ]),
      body: _createMobileMainArea(controller),
      bottomNavigationBar: Obx(() => BottomNavigationBar(
            showSelectedLabels: true,
            showUnselectedLabels: true,
            selectedItemColor: Colors.green,
            unselectedItemColor: Colors.black,
            currentIndex: _moduleNames.indexOf(controller.currentModule.value),
            items: _createNavigationBarItems(),
            onTap: (value) {
              controller.currentModule.value = _moduleNames[value];
            },
          )),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Obx(() => LayoutUtils.isDesktopMode.value
        ? _desktopBuild(context)
        : _mobileBuild(context));
  }
}

class MainPage extends StatelessWidget {
  const MainPage({super.key});

  @override
  Widget build(context) {
    LayoutUtils.setContext(context);
    LayoutUtils.updateMode();
    Common.setPageTitle(context, 'VINNO工时统计系统');
    var pageController = PageController();
    return PageView(
      physics: const NeverScrollableScrollPhysics(),
      controller: pageController,
      children: [
        LoginView(pageController: pageController),
        MainView(pageController: pageController),
      ],
    );
  }
}
