import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:sss/controllers.dart';
import 'package:uuid/uuid.dart';

import 'common.dart';
import 'controls.dart';
import 'dialogs.dart';
import 'entities.dart';
import 'jsonrpc.dart';

class SettingePage extends StatelessWidget {
  const SettingePage({Key? key}) : super(key: key);

  Future _loadSettings(AdminController ac, DomainController dc,
      EmailController ec, MessageTemplateController mc) async {
    EasyLoading.show(status: '正在加载系统设置...');
    try {
      await ec.getEmailSetting();
      await dc.getDomainSetting();
      await ec.getEmailSetting();
      await mc.getMessageTemplate();
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('加载系统设置失败: $e');
    }
  }

  Future _addAndUpdatEmailSetting(
      EmailController ec,
      MessageTemplateController mc,
      String host,
      String port,
      String sender,
      String senderPassword,
      MessageTemplate template) async {
    EasyLoading.show(status: '正在更新Email设置...');
    try {
      var settingId = const Uuid().v1();
      if (ec.setting != null && ec.setting!.id != "") {
        settingId = ec.setting!.id;
      }
      var setting = EmailSetting(settingId, host, port, sender, senderPassword);
      await ec.addOrUpdateEmailSetting(setting);
      await mc.addOrUpdateMessageTemplate(template);
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('更新Email设置失败: $e');
    }
  }

  Widget _syncDingtalkBuilder() {
    return SizedBox(
        width: 380,
        child: Row(children: [
          const SizedBox(width: 128, child: Text('同步钉钉组织架构')),
          Expanded(
              child: Container(
                  alignment: Alignment.centerRight,
                  child: SButton(
                      child: const Text('同步'),
                      onPressed: () {
                        syncDingTakk2();
                      })))
        ]));
  }

  Widget _emailBuilder(EmailController ec, MessageTemplateController mc) {
    var hostTec = TextEditingController(text: ec.setting?.host);
    var portTec = TextEditingController(text: ec.setting?.port);
    var senderTec = TextEditingController(text: ec.setting?.sender);
    var senderPasswordTec =
        TextEditingController(text: ec.setting?.senderPassword);
    var template = mc.templates
        .firstWhereOrNull((element) => element.name == 'ReviewMessageTemplate');
    if (template == null) {
      var content = 'Dear, 请打开链接审核加班统计结果： {URL} 并于 {ExpiredTime} 前完成审核和提交。';
      template =
          MessageTemplate(const Uuid().v1(), 'ReviewMessageTemplate', content);
    }
    var contentTec = TextEditingController(text: template.content);
    return Center(
      child: SizedBox(
          width: 380,
          child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Row(
                  children: [
                    const SizedBox(width: 128, child: Text('邮件服务器地址')),
                    Expanded(child: STextField(controller: hostTec))
                  ],
                ),
                const SizedBox(height: 10),
                Row(
                  children: [
                    const SizedBox(width: 128, child: Text('邮件服务器端口')),
                    Expanded(child: STextField(controller: portTec))
                  ],
                ),
                const SizedBox(height: 10),
                Row(
                  children: [
                    const SizedBox(width: 128, child: Text('发送者邮件')),
                    Expanded(child: STextField(controller: senderTec))
                  ],
                ),
                const SizedBox(height: 10),
                Row(
                  children: [
                    const SizedBox(width: 128, child: Text('发送者密码')),
                    Expanded(
                        child: STextField(
                      controller: senderPasswordTec,
                      obscureText: true,
                    ))
                  ],
                ),
                const SizedBox(height: 10),
                Row(
                  children: [
                    const SizedBox(width: 128, child: Text('消息发送模板')),
                    Expanded(
                        child: STextField(controller: contentTec, maxLines: 8)),
                  ],
                ),
                const SizedBox(height: 10),
                Container(
                  alignment: Alignment.centerRight,
                  width: 380,
                  child: SButton(
                      onPressed: () {
                        template?.content = contentTec.text;
                        _addAndUpdatEmailSetting(
                            ec,
                            mc,
                            hostTec.text,
                            portTec.text,
                            senderTec.text,
                            senderPasswordTec.text,
                            template!);
                      },
                      child: const Text('提交')),
                )
              ])),
    );
  }

  Future _updatAdminPassword(AdminController ac, String password) async {
    if (password.length < 6) {
      Dialogs.showErrorDialog('密码长度不能小于6位');
      return;
    }
    EasyLoading.show(status: '正在更新密码...');
    try {
      await ac.updateAdminPasword(password);
      EasyLoading.dismiss();
      EasyLoading.showInfo('密码更新成功', duration: const Duration(seconds: 1));
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('更新密码失败: $e');
    }
  }

  Widget _adminBuilder(AdminController ac) {
    var passwordTec = TextEditingController();
    return SizedBox(
        width: 380,
        child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Row(
                children: [
                  const SizedBox(width: 128, child: Text('管理员密码')),
                  Expanded(
                      child: STextField(
                          controller: passwordTec, obscureText: true)),
                ],
              ),
              const SizedBox(height: 10),
              Container(
                alignment: Alignment.centerRight,
                width: 460,
                height: 40,
                child: SButton(
                    onPressed: () =>
                        {_updatAdminPassword(ac, passwordTec.text)},
                    child: const Text('提交')),
              )
            ]));
  }

  Future _updatDomain(
      DomainController controller, DomainSetting domainSetting) async {
    if (domainSetting.domain.endsWith('/')) {
      Dialogs.showErrorDialog('域名不可以 / 结尾。');
      return;
    }
    EasyLoading.show(status: '正在更新域名...');
    try {
      await controller.addOrUpdateDomainSetting(domainSetting);
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('更新域名失败: $e');
    }
  }

  Widget _domaintBuilder(DomainController controller) {
    var domain = controller.setting == null ? '' : controller.setting!.domain;
    var domainNameController = TextEditingController(text: domain);
    return Center(
      child: SizedBox(
          width: 380,
          child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Row(
                  children: [
                    const SizedBox(width: 128, child: Text('域名地址')),
                    Expanded(
                        child: STextField(controller: domainNameController)),
                  ],
                ),
                const SizedBox(height: 20),
                Container(
                  alignment: Alignment.centerRight,
                  width: 380,
                  child: SButton(
                      onPressed: () {
                        var domain = controller.setting;
                        if (domain == null) {
                          domain = DomainSetting(
                              const Uuid().v1().toLowerCase(),
                              domainNameController.text);
                        } else {
                          domain.domain = domainNameController.text;
                        }
                        _updatDomain(controller, domain);
                      },
                      child: const Text('提交')),
                )
              ])),
    );
  }

  Widget _createCustomDivider(String title) {
    return Row(children: [
      const SizedBox(width: 8),
      Text(
        title,
        style: const TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
      ),
      const SizedBox(width: 8),
      Expanded(child: Container(height: 1, color: Colors.grey)),
      const SizedBox(width: 8)
    ]);
  }

  Future syncDingTalk() async {
    EasyLoading.show(status: '正在同步数据，请等待...');
    try {
      await JsonRpc()
          .call(Common.host, 'DingTalkService', 'SyncDepartments', []);
      await JsonRpc()
          .call(Common.host, 'DingTalkService', 'SyncEmployees', [1]);
      EasyLoading.dismiss();
      EasyLoading.showInfo('同步成功!');
      await Future.delayed(const Duration(seconds: 1));
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('同步数据失败:\n${e.toString()}');
    }
  }

  Future syncDingTakk2() async {
    var progress = 0;
    var progressValue = progress / 100.0;
    await EasyLoading.showProgress(progressValue,
        status: '正在同步钉钉员工列表数据...$progress%');
    try {
      var progressId = await Common.createProgress();
      JsonRpc().call(
          Common.host, 'DingTalkService', 'SyncEmployeess2', [progressId]);
      while (progress < 100) {
        progress = await Common.getProgress(progressId);
        progressValue = progress / 100.0;
        await EasyLoading.showProgress(progressValue,
            status: '正在同步钉钉员工列表数据...$progress%');
        await Future.delayed(const Duration(seconds: 1));
      }
      await Common.deleteProgress(progressId);
      await EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('同步钉钉员工列表数据失败: ${e.toString()}');
    }
  }

  @override
  Widget build(context) {
    var ac = Get.find<AdminController>();
    var ec = Get.find<EmailController>();
    var mc = Get.find<MessageTemplateController>();
    var dc = Get.find<DomainController>();

    return FutureBuilder(
        future: _loadSettings(ac, dc, ec, mc),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return Container(
              padding: const EdgeInsets.all(8),
              alignment: Alignment.topLeft,
              child: SingleChildScrollView(
                controller: ScrollController(),
                child: Column(
                  children: [
                    const SizedBox(height: 10),
                    _createCustomDivider('安全'),
                    _adminBuilder(ac),
                    const SizedBox(height: 10),
                    _createCustomDivider('数据'),
                    _syncDingtalkBuilder(),
                    const SizedBox(height: 10),
                    _createCustomDivider('域名'),
                    _domaintBuilder(dc),
                    const SizedBox(height: 10),
                    _createCustomDivider('邮件'),
                    _emailBuilder(ec, mc),
                  ],
                ),
              ),
            );
          } else {
            return const SizedBox();
          }
        });
  }
}
