import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flutter_number_picker/flutter_number_picker.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:sss/common.dart';
import 'package:sss/layout.dart';
import 'package:sss/viewmodels.dart';

import 'controllers.dart';
import 'controls.dart';
import 'dialogs.dart';

class StatisticsModule extends StatelessWidget {
  const StatisticsModule({Key? key}) : super(key: key);

  @override
  Widget build(context) {
    StatisticsController controller = Get.find<StatisticsController>();
    return Column(
      children: [
        Expanded(child: _createStatisticsTablesListView(controller)),
        Container(color: Colors.grey, height: 0.5),
        SizedBox(
          height: 64,
          child: _createOperationBar(controller),
        ),
      ],
    );
  }

  Future _addStatisticsTable(StatisticsController controller, String title,
      int requiredReachedCount, DateTime start, DateTime end) async {
    if (title == '') {
      Dialogs.showErrorDialog('请输入统计表标题。');
      return;
    }
    if (!end.isAfter(start)) {
      Dialogs.showErrorDialog('结束时间不能早于开始时间。');
      return;
    }
    if (!start.add(const Duration(days: 31)).isAfter(end)) {
      Dialogs.showErrorDialog('时间段不能超过31天。');
      return;
    }
    var progress = 0;
    var progressValue = progress / 100.0;
    await EasyLoading.showProgress(progressValue,
        status: '正在创建统计表...$progress%');
    try {
      var startStr = DateFormat('yyyy-MM-dd HH:mm:ss').format(start);
      var endStr = DateFormat('yyyy-MM-dd HH:mm:ss').format(end);

      var progressId = await Common.createProgress();
      controller.addStatisticsTable2(
          title, startStr, endStr, requiredReachedCount, progressId);
      while (progress < 100) {
        progress = await Common.getProgress(progressId);
        progressValue = progress / 100.0;
        await EasyLoading.showProgress(progressValue,
            status: '正在创建统计表...$progress%');
        await Future.delayed(const Duration(seconds: 1));
      }
      await Common.deleteProgress(progressId);
      await EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('创建统计表失败: ${e.toString()}');
    }
  }

  Widget _createSelectDayView(PageController pageController, Rx<DateTime> day) {
    return Column(
      children: [
        Row(
          children: [
            SIconButton(
                toolTip: '返回',
                onPressed: () => {pageController.jumpToPage(0)},
                icon: const Icon(Icons.arrow_back)),
            const SizedBox(
              width: 4,
            ),
            const Text('选择日期'),
          ],
        ),
        const SizedBox(height: 4),
        Expanded(
            child: CalendarDatePicker(
                initialDate: day.value,
                firstDate: day.value.add(const Duration(days: -365)),
                lastDate: day.value.add(const Duration(days: 365)),
                onDateChanged: (value) {
                  day.value = value;
                  pageController.animateToPage(0,
                      duration: const Duration(milliseconds: 200),
                      curve: Curves.linear);
                }))
      ],
    );
  }

  Widget _createNewStatisticsTableView(
      PageController pageController,
      TextEditingController titleController,
      Rx<DateTime> start,
      Rx<DateTime> end,
      RxInt requiredReachedCount) {
    return Column(
      children: [
        Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              const Text(
                '标题',
              ),
              const SizedBox(height: 4),
              STextField(
                controller: titleController,
              ),
            ]),
        const SizedBox(height: 16),
        Row(children: [
          const SizedBox(
            width: 100,
            child: Text('需达标次数'),
          ),
          Expanded(
              child: Container(
            alignment: Alignment.centerRight,
            height: 48,
            child: CustomNumberPicker(
              valueTextStyle:
                  const TextStyle(fontFamily: 'Segoe UI', fontSize: 18),
              initialValue: requiredReachedCount.value,
              maxValue: 21,
              minValue: 0,
              step: 1,
              shape: OutlineInputBorder(
                  //未选中时候的颜色
                  borderRadius: BorderRadius.circular(4.0),
                  borderSide: const BorderSide(
                      color: Color.fromARGB(255, 225, 227, 233), width: 2)),
              onValue: (value) {
                requiredReachedCount.value = value as int;
              },
            ),
          ))
        ]),
        const SizedBox(height: 10),
        Row(
          children: [
            const SizedBox(
              width: 100,
              child: Text('开始时间'),
            ),
            Expanded(
                child: Container(
              alignment: Alignment.centerRight,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Obx(() => Text(DateFormat('yyyy-MM-dd').format(start.value))),
                  const SizedBox(
                    width: 8,
                  ),
                  SIconButton(
                      toolTip: '选择时间',
                      onPressed: () => {pageController.jumpToPage(1)},
                      icon: const Icon(Icons.arrow_forward_ios))
                ],
              ),
            ))
          ],
        ),
        const SizedBox(height: 10),
        Row(
          children: [
            const SizedBox(
              width: 100,
              child: Text('结束时间'),
            ),
            Expanded(
                child: Container(
              alignment: Alignment.centerRight,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Obx(() => Text(DateFormat('yyyy-MM-dd').format(end.value))),
                  const SizedBox(
                    width: 8,
                  ),
                  SIconButton(
                      toolTip: '选择时间',
                      onPressed: () => {pageController.jumpToPage(2)},
                      icon: const Icon(Icons.arrow_forward_ios))
                ],
              ),
            ))
          ],
        ),
      ],
    );
  }

  void _showCreateStatisticsTableDialog(StatisticsController controller) {
    var now = DateTime.now();
    PageController pageController = PageController();
    TextEditingController titleController = TextEditingController();
    RxInt requiredReachedCount = RxInt(12);
    var lastMonth = DateUtils.addMonthsToMonthDate(now, -1);
    Rx<DateTime> start = Rx(DateUtils.addDaysToDate(lastMonth, 25));
    Rx<DateTime> end = Rx(now);
    Dialogs.showChildDialog2(
        SizedBox(
          width: 360,
          height: 320,
          child: Column(
            children: [
              const Text('新建统计', style: TextStyle(fontSize: 26)),
              const Divider(),
              const SizedBox(height: 16),
              Expanded(
                  child: PageView(
                physics: const NeverScrollableScrollPhysics(),
                controller: pageController,
                children: [
                  _createNewStatisticsTableView(pageController, titleController,
                      start, end, requiredReachedCount),
                  _createSelectDayView(pageController, start),
                  _createSelectDayView(pageController, end),
                ],
              )),
            ],
          ),
        ),
        actions: [
          SButton(
              onPressed: () => {
                    _addStatisticsTable(controller, titleController.text,
                        requiredReachedCount.value, start.value, end.value)
                  },
              child: const Text('确定')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消'))
        ]);
  }

  Widget _createOperationBar(StatisticsController controller) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        SIconButton(
            onPressed: () => _showCreateStatisticsTableDialog(controller),
            icon: const Icon(Icons.add),
            toolTip: '新建'),
        const SizedBox(
          width: 20,
        ),
        SIconButton(
            onPressed: () => _loadStatisticsTables(controller),
            icon: const Icon(Icons.refresh),
            toolTip: '刷新'),
      ],
    );
  }

  Future _deleteStatisticsTable(StatisticsController sc, String id) async {
    EasyLoading.show(status: '正在删除统计记录...');
    try {
      await sc.deleteStatisticsTable(id);
      EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('删除统计记录失败: ${e.toString()}');
    }
  }

  void _showDeleteStatisticsTableDialog(StatisticsController controller,
      StatisticsTableViewModel statisticsTable) {
    Dialogs.showConfirmDialog('确定删除此统计记录？',
        () => {_deleteStatisticsTable(controller, statisticsTable.data.id)});
  }

  Future _openStatistcsTable(StatisticsController controller,
      StatisticsTableViewModel statisticsTable) async {
    EasyLoading.show(status: '正在加载员工统计列表...');
    try {
      await controller.selectTable(statisticsTable.data);
      EasyLoading.dismiss();
      await Get.toNamed('/viewstatistcs',
          arguments: {"controller": controller});
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('加载员工统计列表失败: ${e.toString()}');
    }
  }

  Future _loadStatisticsTables(StatisticsController controller) async {
    EasyLoading.show(status: '正在加载统计列表...');
    try {
      await controller.loadStatisticsTables();
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('加载统计列表失败: $e');
    }
  }

  Widget _createIsEmployeeCountFlag(int employeeCount) {
    return Container(
        width: 48,
        height: 20,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.black, width: 1),
          color: Colors.white,
          borderRadius: const BorderRadius.all(Radius.circular(4)),
        ),
        padding: const EdgeInsets.all(2),
        child: Row(
          children: [
            const SizedBox(
              width: 2,
            ),
            const Icon(
              Icons.person,
              size: 12,
            ),
            Expanded(
                child: Text('$employeeCount',
                    textAlign: TextAlign.center,
                    style: const TextStyle(fontSize: 10)))
          ],
        ));
  }

  Widget _createStatisticsTableView(StatisticsController controller,
      StatisticsTableViewModel statisticsTable) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        CircleAvatar(child: Text('${statisticsTable.data.createTime.month}')),
        const SizedBox(width: 8),
        Expanded(
            child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Row(children: [
              Text(statisticsTable.data.title),
            ]),
            const SizedBox(
              height: 4,
            ),
            Row(
              children: [
                Text(
                  DateFormat('yyyy-MM-dd HH:mm:ss')
                      .format(statisticsTable.data.createTime),
                  style: const TextStyle(color: Colors.grey, fontSize: 12),
                ),
                const SizedBox(width: 8),
                _createIsEmployeeCountFlag(
                    statisticsTable.data.workingDayStatisticsIds.length)
              ],
            )
          ],
        )),
        SizedBox(
            width: 40,
            child: Row(children: [
              SIconButton(
                  toolTip: '删除',
                  onPressed: () => _showDeleteStatisticsTableDialog(
                      controller, statisticsTable),
                  icon: const Icon(
                    Icons.delete,
                    color: Colors.red,
                  ))
            ]))
      ],
    );
  }

  Widget _createDesktopStatisticsItemView(StatisticsController controller,
      StatisticsTableViewModel statisticsTable) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      onHover: (event) => statisticsTable.isHovering.value = true,
      onExit: (event) => statisticsTable.isHovering.value = false,
      child: Obx(
        () => statisticsTable.isHovering.value
            ? Container(
                margin: const EdgeInsets.all(4),
                padding: const EdgeInsets.all(8),
                decoration: BoxDecoration(
                  border: Border.all(
                      color: const Color.fromARGB(255, 233, 234, 239),
                      width: 2),
                  color: const Color.fromARGB(255, 242, 242, 242),
                  borderRadius: const BorderRadius.all(Radius.circular(2)),
                ),
                child: _createStatisticsTableView(controller, statisticsTable))
            : Container(
                margin: const EdgeInsets.all(4),
                padding: const EdgeInsets.all(8),
                child: _createStatisticsTableView(controller, statisticsTable)),
      ),
    );
  }

  Widget _createStatisticsTablesListView(StatisticsController controller) {
    return FutureBuilder(
        future: _loadStatisticsTables(controller),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return Obx(
              () => ListView.builder(
                controller: ScrollController(),
                padding: EdgeInsets.zero,
                itemCount: controller.statisticsTables.value.length,
                itemExtent: 84,
                itemBuilder: (BuildContext context, int index) {
                  var statisticsTable =
                      controller.statisticsTables.value[index];
                  return Column(
                    children: [
                      Expanded(
                          child: GestureDetector(
                              onTap: () {
                                _openStatistcsTable(
                                    controller, statisticsTable);
                              },
                              child: LayoutUtils.isDesktopMode.value
                                  ? _createDesktopStatisticsItemView(
                                      controller, statisticsTable)
                                  : Container(
                                      color: Colors.white10,
                                      padding: const EdgeInsets.all(8),
                                      child: _createStatisticsTableView(
                                          controller, statisticsTable)))),
                      Container(
                        color: Colors.grey,
                        height: 0.5,
                      )
                    ],
                  );
                },
              ),
            );
          } else {
            return Container();
          }
        });
  }
}
